"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getJsonArray = exports.mergeCsvFilesToJsonArray = void 0;
const fs_1 = require("fs");
const csv = require("csvtojson");
const DEFAULT_FILE_ENCODING = "utf8";
function writeOutputFile(options, jsonObj) {
    const dataBuffer = Buffer.from(JSON.stringify(jsonObj));
    const destFile = `${options.outputDir}/${options.outputFileName}.json`;
    const encoding = options.encoding || DEFAULT_FILE_ENCODING;
    (0, fs_1.writeFile)(destFile, dataBuffer, encoding, function (err) {
        if (err)
            return console.log("multiple-csv-merge-to-json ERROR writing output file : ", err);
        console.log("multiple-csv-merge-to-json SUCCESS file written");
        console.log("multiple-csv-merge-to-json SUCCESS lines written in file %s : %s", destFile, Object.keys(jsonObj).length);
    });
}
function readExistingJsonDataArray(options) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const encoding = options.encoding || DEFAULT_FILE_ENCODING;
            const data = yield (0, fs_1.readFileSync)(`${options.outputDir}/${options.outputFileName}.json`, encoding);
            if (typeof data === "string")
                return JSON.parse(data);
        }
        catch (error) {
            console.log("multiple-csv-merge-to-json ERROR reading file : ", error);
            console.log("multiple-csv-merge-to-json ERROR reading file, file is generated by mergeCsvFilesToJsonArray first ?  ");
        }
        return [];
    });
}
function generateArrayOfJSONfromCSV(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const filesNames = options.inputFileNameList;
        const filesToImport = filesNames.map((fileName) => `${options.inputDir}/${fileName}`);
        let filesExist = true;
        for (const file of filesToImport) {
            const fileExists = yield (0, fs_1.existsSync)(file);
            if (!fileExists) {
                console.log("multiple-csv-merge-to-json ERROR file not found", file);
                filesExist = false;
                break;
            }
        }
        if (filesExist) {
            return Promise.all(filesToImport.map((file) => __awaiter(this, void 0, void 0, function* () {
                console.log("multiple-csv-merge-to-json importing file :", file);
                return csv({ delimiter: options.columnDelimiter }).fromFile(file);
            })));
        }
        return Promise.reject(new Error("At least one file given in options does not exists."));
    });
}
function objectMatchesSearchKeys(objectKeys, dataObject, searchObject) {
    for (const key of objectKeys) {
        if (new String(dataObject[key])
            .toUpperCase()
            .normalize("NFD")
            .replace(/[\u0300-\u036f]/g, "") !==
            new String(searchObject[key])
                .toUpperCase()
                .normalize("NFD")
                .replace(/[\u0300-\u036f]/g, ""))
            return false;
    }
    return true;
}
function mergeObjects(existingObject, newObject) {
    let updatedObject = Object.assign({}, existingObject);
    for (const key in newObject) {
        updatedObject[key] = new String(newObject[key])
            .toUpperCase()
            .normalize("NFD")
            .replace(/[\u0300-\u036f]/g, "");
    }
    return updatedObject;
}
function updateData(objectKeys, existingData, newData) {
    let updatedData = [...existingData];
    for (const data of newData) {
        const indexFound = existingData.findIndex((_existingData) => objectMatchesSearchKeys(objectKeys, _existingData, data));
        if (indexFound >= 0) {
            const mergedObject = mergeObjects(existingData[indexFound], data);
            if (existingData[indexFound].updated) {
                updatedData.push(mergedObject);
            }
            else {
                existingData[indexFound] = Object.assign(Object.assign({}, existingData[indexFound]), { updated: true });
                updatedData[indexFound] = mergedObject;
            }
        }
        else {
            updatedData.push(data);
        }
    }
    return updatedData;
}
function aggregateData(dataList, groupByOptions) {
    let aggregatedArray = [];
    const dataMap = new Map();
    for (const data of dataList) {
        const key = data[groupByOptions.groupByKey];
        const existingDataByKey = dataMap.get(key);
        const object = Object.assign({}, data);
        delete object[groupByOptions.groupByKey];
        if (existingDataByKey) {
            existingDataByKey.push(object);
            dataMap.set(key, existingDataByKey);
        }
        else {
            dataMap.set(key, [object]);
        }
    }
    const mapKeys = dataMap.keys();
    let keyObject = mapKeys.next();
    //Convert map to Object
    const newArray = [];
    while (!keyObject.done) {
        const _key = keyObject.value;
        let _object = {
            [groupByOptions.groupByKey]: _key,
            [groupByOptions.groupedArrayProperty]: dataMap.get(_key),
        };
        aggregatedArray.push(_object);
        keyObject = mapKeys.next();
    }
    return aggregatedArray;
}
function groupByData(dataList, groupByKey) {
    let aggregatedArray = [];
    aggregatedArray = aggregateData(dataList, groupByKey);
    return aggregatedArray;
}
function mergeCsvFilesToJsonArray(options) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const filesDataImported = yield generateArrayOfJSONfromCSV(options);
            console.log("multiple-csv-merge-to-json number of files to import : ", filesDataImported.length);
            let outputData = [{}];
            if (filesDataImported.length > 0) {
                console.log("multiple-csv-merge-to-json reading file at index 0");
                outputData = filesDataImported[0];
                console.log("multiple-csv-merge-to-json lines in buffer END ", outputData.length);
                filesDataImported.slice(1).forEach((fileData, index) => {
                    console.log("multiple-csv-merge-to-json reading file at index %s", index + 1);
                    outputData = updateData(options.inputKeys, outputData, fileData);
                    console.log("multiple-csv-merge-to-json lines in buffer END ", outputData.length);
                });
            }
            if (options.groupBy) {
                const groupedObject = groupByData(outputData, options.groupBy);
                writeOutputFile(options, groupedObject);
            }
            else {
                writeOutputFile(options, outputData);
            }
        }
        catch (error) {
            console.log("multiple-csv-merge-to-json ERROR mergeCsvFilesToJsonArray", error);
            throw error;
        }
    });
}
exports.mergeCsvFilesToJsonArray = mergeCsvFilesToJsonArray;
function getJsonArray(options) {
    return __awaiter(this, void 0, void 0, function* () {
        return readExistingJsonDataArray(options);
    });
}
exports.getJsonArray = getJsonArray;
